import { Component, OnInit } from '@angular/core';
import { Validators, FormControl, FormGroup, FormBuilder } from '@angular/forms';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Fileman Common
import { FilemanSessionService } from '@fileman/index-service';
// ------------------

// Load other
import { FtpSiteDescriptor, FtpDirectoryDescriptor, FtpDirectory, FileLocation, FtpSite } from '@fileman/index-domain';
import { ReferenceCode } from '@fileman/refcode/domain/reference-code';
//  -----------------

@Component({
    templateUrl: './directories.component.html'
})

export class DirectoriesComponent extends PermissionComponent implements OnInit {

    // grid details
    //  ----------------------------------
    sites: FtpSiteDescriptor[] = [];
    siteSelection: FtpSiteDescriptor = undefined;
    site: FtpSite = new FtpSite();

    directions: ReferenceCode[] = [];
    transferTypes: ReferenceCode[] = [];

    refCodesDirectionsActive: ReferenceCode[] = [];
    refCodesTransferTypesActive: ReferenceCode[] = [];

    directories: FtpDirectoryDescriptor[] = [];
    directory: FtpDirectory = new FtpDirectory();
    //  ----------------------------------

    // validation  
    form: FormGroup;
    submitted: boolean;
    output = '';
    // ----------------------------------

    // session values
    sessionSiteId = null;
    sessionDirectoryId = null;
    // ----------------------------------

    isFileLocations = false;

    // load handlers
    // -----------------------------------
    constructor(baseService: BaseService, private fb: FormBuilder, private sessionService: FilemanSessionService) {
        super(baseService);

        // Check permission on function calls in save method
        this.baseService.checkPermission('FTPDirectory');
        // -------------------------------------------------------------------
    }

    ngOnInit() {
        // Call to Permission Component --> componentInit();
        super.componentInit();
        
        // Retrieve values from session
        this.sessionSiteId = +this.sessionService.getSessionIdWithKey('SiteTo');
        this.sessionDirectoryId = +this.sessionService.getSessionIdWithKey('DirectoryTo');
        // -----------------------------------------------

        // form validation
        this.form = new FormGroup({
            'id': new FormControl({ disabled: true }),
            'ftpSiteId': new FormControl({}, Validators.required),
            'directionId': new FormControl({}, Validators.compose([Validators.required])),
            'transferTypeId': new FormControl({}, Validators.compose([Validators.required])),
            'localDir': new FormControl({}, Validators.compose([Validators.required, Validators.maxLength(400)])),
            'localDirTest': new FormControl({}, Validators.compose([Validators.required, Validators.maxLength(400)])),
            'remoteDir': new FormControl({}, Validators.compose([Validators.required, Validators.maxLength(400)])),
            'remoteDirTest': new FormControl({}, Validators.compose([Validators.required, Validators.maxLength(400)])),
            'archiveDir': new FormControl({}, Validators.maxLength(400)),
            'archiveDirTest': new FormControl({}, Validators.maxLength(400)),
            'isActive': new FormControl({}, [Validators.required])
        });

        this.determineFtpSiteDescriptorForSiteId(this.sessionSiteId); 

        // If there is a site session  - reload for site
        if (this.sessionDirectoryId || this.sessionDirectoryId !== 0) {            
            this.getGridDetails(this.sessionDirectoryId);
        } else {
            // if there is a sit only
            if (this.sessionSiteId || this.sessionSiteId !== 0) {
                this.btn_new();
                this.form.get('ftpSiteId').setValue(this.sessionSiteId);
            }

        }
        
        // Call to getSitesList();
        this.getSitesList();

        // Call to getDirectionsList();
        this.getDirectionsList();

        // Call to getTransferTypesList();
        this.getTransferTypesList();

        // Call to local getGridList();
        this.getGridList();

    }
    // -----------------------------------

    // basic button events
    // -----------------------------------
    btn_done() {
        this.detail = false;
    }

    btn_back() {
        this.edit = false;
        this.form.reset();
        this.baseService.showAudit(false);
        
        // If there is a site session  - go back to sites
        if (this.sessionDirectoryId || this.sessionDirectoryId !== null) {
            this.sessionService.setSession(0, 'Directory', 'DirectoryTo');
            this.baseService.loadMenu('fileman/sites');
        }
           
    }

    btn_new() {
        this.edit = true;
        this.form.reset();
        this.baseService.showAudit(false);

        // Reset all items to null - empty inputs
        // ---------------------------------------
        this.directory = new FtpDirectory();
        this.directory.isActive = true;
        if (this.site) {
            this.directory.ftpSite = this.site;
            this.form.get('ftpSiteId').setValue(this.directory.ftpSite.id);
        }
        // ---------------------------------------

        this.isFileLocations = false;
    }

    btn_save() {
        this.saveDetails();
    }

    btn_select_row(index: number) {
        
        // Audit information
        this.baseService.showAudit(true);
        this.baseService.auditTable = this.directories[index].domainName
        this.baseService.auditId = this.directories[index].id;
        // ------------------------------------------------------------------

        this.getGridDetails(this.directories[index].id);
    }

    btn_show_row(index: number) {
        this.directory = this.directories[index] as FtpDirectory;
        this.output = JSON.stringify(this.directory);
        this.output = this.output.replace(/":{"/g, '</strong><i><br></i><strong>');
        this.output = this.output.replace(/":"/g, '</strong><i>');
        this.output = this.output.replace(/",/g, '</i><br>');
        this.output = this.output.replace(/,/g, '</i><br>');
        this.output = this.output.replace(/"id":/g, "<strong>id</strong><i>");
        this.output = this.output.replace(/":/g, '</strong><i>');
        this.output = this.output.replace(/"/g, "<strong>");
        this.output = this.output.replace(/{/g, '<br>');
        this.output = this.output.replace(/}/g, '</i>') + '</i><br>';
        this.detail = true;
    }

    determineFtpSiteDescriptorForSiteId(siteId: number) {
        for (let i=0; i<this.sites.length; i++) {
            if (this.sites[i].id === siteId) {
                return this.sites[i];
            }
        }
        return undefined;
    }

    onSelectSite(siteId: string) {
        if (siteId && siteId != null && siteId !== 'undefined') {
            this.siteSelection = this.determineFtpSiteDescriptorForSiteId(+siteId);
            this.getGridList(+siteId);
        } else {
            this.siteSelection = undefined;
            this.getGridList(null);
        }
    }
    // -----------------------------------

    // Data calls
    // -----------------------------------
    getSitesList() {
        this.baseService.getData('GET', 'locations/ftpSites?size=600&sort=ipAddress,username')
            .subscribe(
                (data) => {
                    this.sites = data.json().content as FtpSiteDescriptor[];
                    this.nodata = false;
                }
            );
    }

    getDirectionsList() {
        this.baseService.getData('GET', 'referenceCodes/domain/DIRECTION')
            .subscribe(
                (data) => {
                    this.directions = data.json().content as ReferenceCode[];
                    
                    // create active list only
                    for (let i = 0; i < this.directions.length; i++) {
                        if (this.directions[i].isActive) {
                            this.refCodesDirectionsActive.push(this.directions[i]);
                        }
                    }
                    // --------------------------------------------------------

                    this.nodata = false;
                }
            );
    }

    getTransferTypesList() {
        this.baseService.getData('GET', 'referenceCodes/domain/TRANSFER_TYPE')
            .subscribe(
                (data) => {
                    this.transferTypes = data.json().content as ReferenceCode[];
                    
                    // create active list only
                    for (let i = 0; i < this.transferTypes.length; i++) {
                        if (this.transferTypes[i].isActive) {
                            this.refCodesTransferTypesActive.push(this.transferTypes[i]);
                        }
                    }
                    // --------------------------------------------------------

                    this.nodata = false;
                }
            );
    }

    getGridList(siteId?: number) {
        let endpoint = 'locations/ftpDirectories?size=600&sort=localDir';
        let sitePresent = false;
        if (siteId && siteId != null) {
            endpoint = 'locations/ftpSites/' + siteId + '?size=600&sort=localDir';
            sitePresent = true;
        }
        this.baseService.getData('GET', endpoint)
            .subscribe(
                (data) => {
                    if (sitePresent) {
                        this.site = data.json();
                        this.directories = this.site.ftpDirectories as FtpDirectoryDescriptor[];
                        this.siteSelection = this.determineFtpSiteDescriptorForSiteId(this.site.id);
                        this.form.get('ftpSiteId').setValue(this.site.id);
                    } else {
                        this.directories = data.json().content as FtpDirectoryDescriptor[];
                        this.site = undefined;
                    }
                    this.nodata = false;
                }
            );
    }

    determineIsFileLocations(fileLocs: FileLocation[]): boolean {
        return (fileLocs && fileLocs.length > 0);
    }

    determineModelSiteId(): number {
        return (this.directory.ftpSite) ? this.directory.ftpSite.id : ((this.site) ? this.site.id : undefined);
    }

    determineModelDirectionId(): string {
        return (this.directory.direction) ? this.directory.direction.value : undefined;
    }

    determineModelTransferTypeId(): string {
        return (this.directory.transferType) ? this.directory.transferType.value : undefined;
    }

    getGridDetails(dirId?: number) {
        if (dirId && dirId != null) {
            this.baseService.getData('GET', 'locations/ftpDirectories/' + dirId)
                .subscribe(
                    (data) => {
                        this.directory = data.json() as FtpDirectory;
                        
                       
                        // Audit information
                        this.baseService.showAudit(true);
                        this.baseService.auditTable = this.directory.domainName
                        this.baseService.auditId = this.directory.id;
                        // ------------------------------------------------------------------                

                        this.form.get('id').setValue(this.directory.id);
                        this.form.get('ftpSiteId').setValue(this.directory.ftpSite.id);
                        this.form.get('localDir').setValue(this.directory.localDir);
                        this.form.get('localDirTest').setValue(this.directory.localDirTest);
                        this.form.get('remoteDir').setValue(this.directory.remoteDir);
                        this.form.get('remoteDirTest').setValue(this.directory.remoteDirTest);
                        this.form.get('archiveDir').setValue(this.directory.archiveDir);
                        this.form.get('archiveDirTest').setValue(this.directory.archiveDirTest);
                        this.form.get('isActive').setValue(this.directory.isActive);
                        
                        if (this.directory.direction.id) {
                            this.form.get( 'directionId').setValue(this.directory.direction.id);
                        } else {
                            this.form.get( 'directionId').setValue('undefined');
                        }

                        if (this.directory.transferType.id) {
                            this.form.get( 'transferTypeId').setValue(this.directory.transferType.id);
                        } else {
                            this.form.get( 'transferTypeId').setValue('undefined');
                        }

                        this.nodata = false;
                        this.edit = true;
                        this.isFileLocations = this.determineIsFileLocations(this.directory.fileLocations);
                    }
                );
        }
    }
    
    populateDomainDirectoryFromFormValuesAndIds(): void {
        this.directory.localDir = this.form.get('localDir').value;
        this.directory.localDirTest = this.form.get('localDirTest').value;
        this.directory.remoteDir = this.form.get('remoteDir').value;
        this.directory.remoteDirTest = this.form.get('remoteDirTest').value;
        this.directory.archiveDir = this.form.get('archiveDir').value;
        this.directory.archiveDirTest = this.form.get('archiveDirTest').value;
        this.directory.isActive = this.form.get('isActive').value;

        for (let i = 0; i < this.sites.length; i++) {
            if (this.sites[i].id === +this.form.get('ftpSiteId').value) {
                this.directory.ftpSite = this.sites[i]
                break;
            }
        }

        for (let i = 0; i < this.refCodesDirectionsActive.length; i++) {
            if (this.refCodesDirectionsActive[i].value === this.form.get('directionId').value) {
                this.directory.direction = this.refCodesDirectionsActive[i];
                break;
            }
        }

        for (let i = 0; i < this.refCodesTransferTypesActive.length; i++) {
            if (this.refCodesTransferTypesActive[i].value === this.form.get('transferTypeId').value) {
                this.directory.transferType = this.refCodesTransferTypesActive[i];
                break;
            }
        }
        
    }

    saveDetails() {
        const method: string = this.baseService.checkMethod(this.directory.id);
        const url: string = 'locations/ftpDirectories' + this.baseService.checkCreate(this.directory.id);
        const idMemory = +this.form.get('ftpSiteId').value;
        
        this.directory = new FtpDirectory();
        this.populateDomainDirectoryFromFormValuesAndIds();
        
        const jsonValue: string = JSON.stringify(this.directory);

        // Update / Create items
        this.baseService.getData(method, url, null /* searchParams */, false /* paging */, jsonValue)
            .subscribe(
                (data) => {
                    this.baseService.handleErrorLog('success', 'Your FTP Directory was saved.', 'Saved');
                    this.directory = data.json() as FtpDirectory;
                    this.getGridList(idMemory);
                    this.siteSelection = this.determineFtpSiteDescriptorForSiteId(idMemory);
                    this.edit = false;
                    this.isFileLocations = this.determineIsFileLocations(this.directory.fileLocations);
                },
                (error: any) => {
                    this.baseService.handleErrorLog('error', 'Saving of the FTP Directory failed!', error);
                }
            );
        // -----------------------------------
    }
    // -----------------------------------
}
