import { Component, OnInit } from '@angular/core';
import { Validators, FormControl, FormGroup, FormBuilder } from '@angular/forms';

// Global Common
import { BaseService } from '@shared/index-service';
import { PermissionComponent } from '@shared/index';
// ------------------

// Fileman Common
import { FilemanSessionService } from '@fileman/index-service';
// ------------------

// Load other
import { FtpSiteDescriptor, FtpSite, FtpDirectoryDescriptor } from '@fileman/index-domain';
import { IPAddressValidator } from '@fileman/index-validator';
//  -----------------

@Component({
    templateUrl: './sites.component.html'
})

export class SitesComponent extends PermissionComponent implements OnInit {

    // grid details
    //  ----------------------------------
    sites: FtpSiteDescriptor[] = [];
    siteDirectory: FtpSite = new FtpSite();
    //  ----------------------------------

    // validation  
    form: FormGroup;
    submitted: boolean;
    output = '';
    // ----------------------------------

    // session values
    sessionSiteId = null;
    // ----------------------------------

    isDirectories = false;

    // load handlers
    // -----------------------------------
    constructor(baseService: BaseService, private fb: FormBuilder, private sessionService: FilemanSessionService) {
        super(baseService);

        // Check permission on function calls in save method
        this.baseService.checkPermission('FTPSite');
        // -------------------------------------------------------------------
    }

    ngOnInit() {
        // call to Permission Component --> componentInit();
        super.componentInit();

        // Retrieve values from session
        this.sessionSiteId = +this.sessionService.getSessionIdWithKey('SiteTo');
        // -----------------------------------------------

        // form validation
        this.form = new FormGroup({
            'id': new FormControl({ disabled: true }),
            'ipAddress': new FormControl({}, Validators.compose([Validators.required, Validators.maxLength(30), IPAddressValidator.validIP])),
            'username': new FormControl({}, Validators.compose([Validators.required, Validators.maxLength(30)])),
            'password': new FormControl({}, Validators.compose([Validators.required, Validators.maxLength(2000)])),
            'purpose': new FormControl({}, Validators.compose([Validators.required, Validators.maxLength(2000)])),
            'sftpProtocol': new FormControl({}, [Validators.required]),
            'isActive': new FormControl({}, [Validators.required])
        });

        // Call to local getGridList();
        this.getGridList();

        // If there is a site session  - reload for site
        if (this.sessionSiteId || this.sessionSiteId != null) {

            this.getGridDetails(this.sessionSiteId);
            
            // Audit information
            //this.baseService.showAudit(true);
            //this.baseService.auditTable = this.siteDirectory.domainName
            //this.baseService.auditId = this.siteDirectory.id;
            // ------------------------------------------------------------------
            
        }
    }
    // -----------------------------------

    // basic button events
    // -----------------------------------
    btn_done() {
        this.detail = false;
    }

    btn_back() {
        this.edit = false;
        this.form.reset();
        this.baseService.showAudit(false);

        // reset session
        if (this.sessionSiteId || this.sessionSiteId != null) {
            this.sessionService.setSession(null, 'Site', 'SiteTo');
            this.sessionService.setSession(null, 'Directory', 'DirectoryTo');
            this.baseService.loadMenu('fileman/sites');
        }
    }

    btn_new() {
        this.edit = true;
        this.form.reset();
        this.baseService.showAudit(false);
        this.isDirectories = false;

        // Reset all items to null - empty inputs
        // ---------------------------------------
        this.siteDirectory = new FtpSite();
        this.siteDirectory.isActive = true;
        this.siteDirectory.sftpProtocol = false;
        // ---------------------------------------
    }

    btn_new_directory() {
        this.sessionService.setSession(+this.form.get('id').value, 'Site', 'SiteTo');
        this.baseService.loadMenu('fileman/directories');
    }

    btn_save() {
        this.saveDetails();
    }

    btn_select_row(index: number) {
        
        // Audit information
        this.baseService.showAudit(true);
        this.baseService.auditTable = this.sites[index].domainName
        this.baseService.auditId = this.sites[index].id;
        // ------------------------------------------------------------------

        this.getGridDetails(this.sites[index].id);
    }

    btn_select_row_directory(directoryId: number) {
        this.sessionService.setSession(+this.form.get('id').value, 'Site', 'SiteTo');
        this.sessionService.setSession(directoryId, 'Directory', 'DirectoryTo');
        this.baseService.loadMenu('fileman/directories');
    }

    btn_show_row(index: number) {
        this.siteDirectory = this.sites[index] as FtpSite;
        this.output = JSON.stringify(this.siteDirectory);
        this.output = this.output.replace(/":{"/g, '</strong><i><br></i><strong>');
        this.output = this.output.replace(/":"/g, '</strong><i>');
        this.output = this.output.replace(/",/g, '</i><br>');
        this.output = this.output.replace(/,/g, '</i><br>');
        this.output = this.output.replace(/"id":/g, "<strong>id</strong><i>");
        this.output = this.output.replace(/":/g, '</strong><i>');
        this.output = this.output.replace(/"/g, "<strong>");
        this.output = this.output.replace(/{/g, '<br>');
        this.output = this.output.replace(/}/g, '</i>') + '</i><br>';
        this.detail = true;
    }
    // -----------------------------------

    // Data calls
    // -----------------------------------
    getGridList() {
        this.baseService.getData('GET', 'locations/ftpSites?size=600&sort=ipAddress,username')
            .subscribe(
                (data) => {
                    this.sites = data.json().content as FtpSiteDescriptor[];
                    this.nodata = false;
                }
            );
    }

    determineIsDirectories(ftpDirs: FtpDirectoryDescriptor[]): boolean {
        return (ftpDirs && ftpDirs.length > 0);
    }

    getGridDetails(siteId?: number) {
        if (siteId && siteId != null) {
            this.baseService.getData('GET', 'locations/ftpSites/' + siteId)
                .subscribe(
                    (data) => {
                        this.siteDirectory = data.json() as FtpSite;

                        this.form.get('id').setValue(this.siteDirectory.id);
                        this.form.get('ipAddress').setValue(this.siteDirectory.ipAddress);
                        this.form.get('username').setValue(this.siteDirectory.username);
                        this.form.get('password').setValue(this.siteDirectory.password);
                        this.form.get('purpose').setValue(this.siteDirectory.purpose);
                        this.form.get('isActive').setValue(this.siteDirectory.isActive);
                        this.form.get('sftpProtocol').setValue(this.siteDirectory.sftpProtocol);

                        this.nodata = false;
                        this.edit = true;
                        this.isDirectories = this.determineIsDirectories(this.siteDirectory.ftpDirectories);
                    }
                );
        }
    }

    saveDetails() {
        const method: string = this.baseService.checkMethod(this.siteDirectory.id);
        const url: string = 'locations/ftpSites' + this.baseService.checkCreate(this.siteDirectory.id);
        const jsonValue: string = JSON.stringify(this.form.value);

        // Update / Create items
        this.baseService.getData(method, url, null /* searchParams */, false /* paging */, jsonValue)
            .subscribe(
                (data) => {
                    this.baseService.handleErrorLog('success', 'Your FTP Site was saved.', 'Saved');
                    this.siteDirectory = data.json() as FtpSite;
                    this.getGridList();
                    this.edit = false;
                    this.isDirectories = this.determineIsDirectories(this.siteDirectory.ftpDirectories);
                },
                (error: any) => {
                    this.baseService.handleErrorLog('error', 'Saving of the FTP Site failed!', error);
                }
            );
        // -----------------------------------
    }
    // -----------------------------------

}
